@@ -1,13 +1,13 @@
 #include "configargs.h"
 
-#define GCCPLUGIN_VERSION_MAJOR   4
-#define GCCPLUGIN_VERSION_MINOR   9
-#define GCCPLUGIN_VERSION_PATCHLEVEL   3
+#define GCCPLUGIN_VERSION_MAJOR   6
+#define GCCPLUGIN_VERSION_MINOR   3
+#define GCCPLUGIN_VERSION_PATCHLEVEL   1
 #define GCCPLUGIN_VERSION  (GCCPLUGIN_VERSION_MAJOR*1000 + GCCPLUGIN_VERSION_MINOR)
 
-static char basever[] = "4.9.3";
-static char datestamp[] = "20150311";
-static char devphase[] = "prerelease";
+static char basever[] = "6.3.1";
+static char datestamp[] = "20170109";
+static char devphase[] = "";
 static char revision[] = "";
 
 /* FIXME plugins: We should make the version information more precise.
